@@ -15,11 +15,6 @@
 
 """Brocade generator."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 from capirca.lib import cisco
 
 
