@@ -206,7 +206,7 @@ class Term(gcp.Term):
       for proto in filtered_protocols:
         # If the protocol name is not supported, use the protocol number.
         if proto not in self._ALLOW_PROTO_NAME:
-          proto = self.PROTO_MAP[proto]
+          proto = str(self.PROTO_MAP[proto])
           logging.info('INFO: Term %s is being rendered using protocol number',
                        self.term.name)
         proto_ports = {'ipProtocol': proto}
@@ -368,6 +368,8 @@ class HierarchicalFirewall(gcp.GCP):
     is_policy_modified = False
     counter = 1
     total_cost = 0
+    policies_max_cost = self._DEFAULT_MAXIMUM_COST
+    previous_max_cost = -1
     for header, terms in pol.filters:
 
       if self._PLATFORM not in header.platforms:
@@ -413,9 +415,13 @@ class HierarchicalFirewall(gcp.GCP):
         raise gcp.HeaderError(
             'Default maximum cost cannot be higher than 65536')
 
-      if api_version == 'beta' and address_family != 'inet':
+      if previous_max_cost != -1 and previous_max_cost != max_cost:
         raise gcp.HeaderError(
-            'Beta compute API does not support IPv6, only inet is supported')
+            'Maximum costs of each policy specified must be equal. '
+            'Unequal costs found: %d and %d' % (previous_max_cost, max_cost))
+
+      policies_max_cost = max_cost
+      previous_max_cost = max_cost
 
       display_name = ApiVersionSyntaxMap.SYNTAX_MAP[api_version]['display_name']
 
@@ -498,14 +504,16 @@ class HierarchicalFirewall(gcp.GCP):
             continue
           for dict_term in rules:
             total_cost += GetRuleTupleCount(dict_term, api_version)
-            if total_cost > max_cost:
-              raise ExceededCostError(
-                  'Policy cost (%d) for %s reached the '
-                  'maximum (%d)' %
-                  (total_cost, policy[display_name], max_cost))
             policy['rules'].append(dict_term)
           counter += len(rules)
 
+    # We want to check the total policy cost, not just per policy.
+    if total_cost > policies_max_cost:
+      raise ExceededCostError(
+          'Policy cost (%d) for %s reached the '
+          'maximum (%d)' %
+          (total_cost, policy[display_name], policies_max_cost))
+
     self.policies.append(policy)
 
     # Do not render an empty rules if no policies have been evaluated.
