@@ -16,10 +16,6 @@
 """SRX generator."""
 # pylint: disable=super-init-not-called
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
 
 import collections
 import copy
@@ -79,7 +75,7 @@ class IndentList(list):
 
   def __init__(self, indent, *args, **kwargs):
     self._indent = indent
-    super(IndentList, self).__init__(*args, **kwargs)
+    super().__init__(*args, **kwargs)
 
   def IndentAppend(self, size, data):
     self.append('%s%s' % (self._indent * size, data))
@@ -104,7 +100,7 @@ class Term(aclgenerator.Term):
              'dscp': 'dscp'}
 
   def __init__(self, term, from_zone, to_zone, expresspath=False, verbose=True):
-    super(Term, self).__init__(term)
+    super().__init__(term)
     self.term = term
     self.from_zone = from_zone
     self.to_zone = to_zone
@@ -115,14 +111,6 @@ class Term(aclgenerator.Term):
 
   def __str__(self):
     """Render config output from this term object."""
-    # Verify platform specific terms. Skip whole term if platform does not
-    # match.
-    if self.term.platform:
-      if 'srx' not in self.term.platform:
-        return ''
-    if self.term.platform_exclude:
-      if 'srx' in self.term.platform_exclude:
-        return ''
     ret_str = IndentList(JuniperSRX.INDENT)
 
     # COMMENTS
@@ -306,7 +294,7 @@ class JuniperSRX(aclgenerator.ACLGenerator):
     self.from_zone = ''
     self.to_zone = ''
     self.addr_book_type = set()
-    super(JuniperSRX, self).__init__(pol, exp_info)
+    super().__init__(pol, exp_info)
 
   def _BuildTokens(self):
     """Build supported tokens for platform.
@@ -314,8 +302,7 @@ class JuniperSRX(aclgenerator.ACLGenerator):
     Returns:
       tuple containing both supported tokens and sub tokens
     """
-    supported_tokens, supported_sub_tokens = super(
-        JuniperSRX, self)._BuildTokens()
+    supported_tokens, supported_sub_tokens = super()._BuildTokens()
 
     supported_tokens |= {'dscp_except',
                          'dscp_match',
@@ -447,6 +434,20 @@ class JuniperSRX(aclgenerator.ACLGenerator):
       new_terms = []
       self._FixLargePolices(terms, filter_type)
       for term in terms:
+        # Only generate the term if it's for the appropriate platform.
+        if term.platform:
+          if self._PLATFORM not in term.platform:
+            continue
+        if term.platform_exclude:
+          if self._PLATFORM in term.platform_exclude:
+            continue
+
+        if term.stateless_reply:
+          logging.warning(
+              "WARNING: Term %s in policy %s>%s is a stateless reply "
+              "term and will not be rendered.", term.name, self.from_zone,
+              self.to_zone)
+          continue
         if set(['established', 'tcp-established']).intersection(term.option):
           logging.debug('Skipping established term %s because SRX is stateful.',
                         term.name)
@@ -505,7 +506,7 @@ class JuniperSRX(aclgenerator.ACLGenerator):
         if self._GLOBAL_ADDR_BOOK in self.addr_book_type:
           for zone in self.addressbook:
             for unused_name, ips in sorted(
-                six.iteritems(self.addressbook[zone])):
+                self.addressbook[zone].items()):
               ips = [i for i in ips]
               if term.source_address == ips:
                 term.source_address = ips
@@ -866,6 +867,12 @@ class JuniperSRX(aclgenerator.ACLGenerator):
         if app_list:
           target.extend(app_list)
 
+
+    if len(done_apps) == 0:
+      target.clear()
+      target.append('delete: applications;')
+      return target
+
     target.extend(apps_set_list)
     target.append('}\n')
     return target
