@@ -239,6 +239,10 @@ class Rule:
         if (proto_name in ["igmp", "sctp", "gre"] and
             proto_name not in options["application"]):
           options["application"].append(proto_name)
+        elif proto_name in ("ah", "esp"):
+          ipsec_app_proto = "ipsec-%s" % proto_name
+          if ipsec_app_proto not in options["application"]:
+            options["application"].append(ipsec_app_proto)
 
     return options, new_term
 
@@ -259,12 +263,51 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
       "sctp",
       "igmp",
       "gre",
+      "ah",
+      "esp",
   ]
   _MAX_RULE_DESCRIPTION_LENGTH = 1024
   _MAX_TAG_COMMENTS_LENGTH = 1023
   _TAG_NAME_FORMAT = "{from_zone}_{to_zone}_policy-comment-{num}"
   _MAX_RULE_SRC_DST_MEMBERS = 65535
 
+  _ABBREVIATION_TABLE = [
+      # Service abbreviations first.
+      ("experiment", "EXP"),
+      ("wifi-radius", "W-R"),
+      ("customer", "CUST"),
+      ("server", "SRV"),
+      # Next, common routing terms
+      ("global", "GBL"),
+      ("google", "GOOG"),
+      ("service", "SVC"),
+      ("router", "RTR"),
+      ("internal", "INT"),
+      ("external", "EXT"),
+      ("transit", "TRNS"),
+      ("management", "MGMT"),
+      # State info
+      ("established", "EST"),
+      ("unreachable", "UNR"),
+      ("fragment", "FRAG"),
+      ("accept", "ACC"),
+      ("discard", "DISC"),
+      ("reject", "REJ"),
+      ("replies", "RPLS"),
+      ("reply", "RPL"),
+      ("request", "REQ"),
+      # ICMP types specific
+      ("inverse", "INV"),
+      ("neighbor", "NBR"),
+      ("discovery", "DSCVR"),
+      ("advertisement", "ADV"),
+      ("solicitation", "SOL"),
+      ("multicast", "MCAST"),
+      ("certification", "CERT"),
+      ("listener", "LSNR"),
+      ("address", "ADDR"),
+  ]
+
   INDENT = "  "
 
   def __init__(self, pol, exp_info):
@@ -554,6 +597,30 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
             else:
               exclude_address_family.append(4)
 
+        # Substitute large IPv6 ranges (/1, /2) with equivalent subnets.
+        # Do this separately from address book building, or during policy
+        # translation to account for both address-objects and no-address-objects
+        if term.source_address:
+          saddr_split = []
+          for saddr in term.source_address:
+            if saddr.version == 6 and 0 < saddr.prefixlen < 3:
+              for saddr2 in saddr.subnets(new_prefix=3):
+                saddr2.parent_token = saddr.parent_token
+                saddr_split.append(saddr2)
+            else:
+              saddr_split.append(saddr)
+          term.source_address = saddr_split
+        if term.destination_address:
+          daddr_split = []
+          for daddr in term.destination_address:
+            if daddr.version == 6 and 0 < daddr.prefixlen < 3:
+              for daddr2 in daddr.subnets(new_prefix=3):
+                daddr2.parent_token = daddr.parent_token
+                daddr_split.append(daddr2)
+            else:
+              daddr_split.append(daddr)
+          term.destination_address = daddr_split
+
         # Build address book for the addresses referenced in the term.
         for addr in term.source_address:
           if addr.version in exclude_address_family:
@@ -606,6 +673,10 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
           for term_icmp_type_name in term.icmp_type:
             if icmp_version == "icmp":
               icmp_app_name = "icmp-%s" % term_icmp_type_name
+              # This is to abbreviate the Application name where possible.
+              # The limit is defined by _TERM_MAX_LENGTH = 31.
+              if len(icmp_app_name) > self._TERM_MAX_LENGTH:
+                icmp_app_name = self.FixTermLength(icmp_app_name, True, True)
               if term_icmp_type_name not in policy.Term.ICMP_TYPE[4]:
                 raise PaloAltoFWBadIcmpTypeError(
                     "term with bad icmp type: %s, icmp_type: %s" %
@@ -613,26 +684,33 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
               term_icmp_type = policy.Term.ICMP_TYPE[4][term_icmp_type_name]
             else:
               icmp_app_name = "icmp6-%s" % term_icmp_type_name
+              # This is to abbreviate the Application name where possible.
+              # The limit is defined by _TERM_MAX_LENGTH = 31.
+              if len(icmp_app_name) > self._TERM_MAX_LENGTH:
+                icmp_app_name = self.FixTermLength(icmp_app_name, True, True)
               if term_icmp_type_name not in policy.Term.ICMP_TYPE[6]:
                 raise PaloAltoFWBadIcmpTypeError(
                     "term with bad icmp type: %s, icmp_type: %s" %
                     (term.name, term_icmp_type_name))
               term_icmp_type = policy.Term.ICMP_TYPE[6][term_icmp_type_name]
-            if icmp_app_name in self.application_refs:
-              # the custom icmp application already exists
-              continue
-            app_entry = {
-                "category": "networking",
-                "subcategory": "ip-protocol",
-                "technology": "network-protocol",
-                "description": icmp_app_name,
-                "default": {
-                    icmp_type_keyword: "%d" % term_icmp_type,
-                },
-                "risk": "%d" % risk_level,
-            }
-            self.application_refs[icmp_app_name] = app_entry
-            self.applications.append(icmp_app_name)
+            if icmp_app_name not in self.application_refs:
+              # the custom icmp application doesn't already exist
+              app_entry = {
+                  "category": "networking",
+                  "subcategory": "ip-protocol",
+                  "technology": "network-protocol",
+                  "description": icmp_app_name,
+                  "default": {
+                      icmp_type_keyword: "%d" % term_icmp_type,
+                  },
+                  "risk": "%d" % risk_level,
+              }
+              self.application_refs[icmp_app_name] = app_entry
+              self.applications.append(icmp_app_name)
+
+            # always add the ICMP application to the term, it either already
+            # existed due to a previous policy, or it was created in the
+            # previous loop.
             if icmp_app_name not in term.pan_application:
               term.pan_application.append(icmp_app_name)
 
