@@ -31,6 +31,7 @@ DEFINITIONS = None
 DEFAULT_DEFINITIONS = './def'
 ACTIONS = set(('accept', 'count', 'deny', 'reject', 'next',
                'reject-with-tcp-rst'))
+PROTOS_WITH_PORTS = frozenset(('tcp', 'udp', 'udplite', 'sctp'))
 _FLEXIBLE_MATCH_RANGE_ATTRIBUTES = {'byte-offset',
                                     'bit-offset',
                                     'bit-length',
@@ -123,7 +124,7 @@ class InvalidTermTTLValue(Error):
 
 
 class MixedPortandNonPortProtos(Error):
-  """Error when TCP or UDP are used with protocols that do not use ports."""
+  """Error when protocols that use ports are mixed with protocols that do not"""
 
 
 def TranslatePorts(ports, protocols, term_name):
@@ -327,6 +328,7 @@ class Term:
     logging: VarType.LOGGING
     log_name: VarType.LOG_NAME
     next-ip: VarType.NEXT_IP
+    port-mirror: VarType.PORT_MIRROR
     qos: VarType.QOS
     pan-application: VarType.PAN_APPLICATION
     policer: VarType.POLICER
@@ -454,6 +456,7 @@ class Term:
     self.destination_prefix_except = []
     self.inactive = False
     self.encapsulate = None
+    self.port_mirror = None
     # srx specific
     self.vpn = None
     # gce specific
@@ -655,6 +658,9 @@ class Term:
           return False
       else:
         return False
+    if self.port_mirror:
+      if not other.port_mirror:
+        return False
     if self.icmp_type:
       if sorted(self.icmp_type) is not sorted(other.icmp_type):
         return False
@@ -734,6 +740,8 @@ class Term:
       ret_str.append('  owner: %s' % self.owner)
     if self.port:
       ret_str.append('  port: %s' % sorted(self.port))
+    if self.port_mirror:
+      ret_str.append('  port_mirror: %s' % self.port_mirror)
     if self.source_port:
       ret_str.append('  source_port: %s' % sorted(self.source_port))
     if self.destination_port:
@@ -925,6 +933,10 @@ class Term:
     if self.flexible_match_range != other.flexible_match_range:
       return False
 
+    # port_mirror
+    if self.port_mirror != other.port_mirror:
+      return False
+
     return True
 
   def __ne__(self, other):
@@ -1156,6 +1168,8 @@ class Term:
         self.counter = obj
       elif obj.var_type is VarType.ENCAPSULATE:
         self.encapsulate = obj.value
+      elif obj.var_type is VarType.PORT_MIRROR:
+        self.port_mirror = obj.value
       elif obj.var_type is VarType.TRAFFIC_CLASS_COUNT:
         self.traffic_class_count = obj
       elif obj.var_type is VarType.ICMP_TYPE:
@@ -1237,16 +1251,25 @@ class Term:
             'term "%s" has both verbatim and non-verbatim tokens.' % self.name)
     else:
       if (not self.action and not self.routing_instance and not self.next_ip and
-          not self.encapsulate and not self.filter_term):
+          not self.encapsulate and not self.filter_term and
+          not self.port_mirror):
         raise TermNoActionError('no action specified for term %s' % self.name)
       if self.filter_term and self.action:
         raise InvalidTermActionError('term "%s" has both filter and action tokens.' % self.name)
       # have we specified a port with a protocol that doesn't support ports?
-      if self.source_port or self.destination_port or self.port:
-        if not any(proto in self.protocol for proto in ['tcp', 'udp', 'sctp']):
+      protos_no_ports = {p for p in self.protocol if p not in PROTOS_WITH_PORTS}
+      if protos_no_ports != set() and (self.source_port or
+                                       self.destination_port or self.port):
+        if set(self.protocol) - protos_no_ports != set():
+          # This is a more specific error - some protocols support, but not all
+          raise MixedPortandNonPortProtos(
+              'Term %s contains mixed uses of protocols with and without port '
+              'numbers\nProtocols: %s' % (self.name, self.protocol))
+        else:
           raise TermPortProtocolError(
-              'ports specified with a protocol that doesn\'t support ports. '
-              'Term: %s ' % self.name)
+              'ports specified with protocol(s) that don\'t support ports. '
+              'Term: %s Protocols: %s ' % (self.name, protos_no_ports))
+
     # TODO(pmoody): do we have mutually exclusive options?
     # eg. tcp-established + tcp-initial?
 
@@ -1286,12 +1309,6 @@ class Term:
             self.ICMP_TYPE[6]):
           raise TermInvalidIcmpType('Term %s contains an invalid icmp-type:'
                                     '%s' % (self.name, icmptype))
-    proto_copy = [p for p in self.protocol if p != 'tcp' and p != 'udp']
-    if ('tcp'in self.protocol or 'udp' in self.protocol) and proto_copy:
-      if self.source_port or self.destination_port or self.port:
-        raise MixedPortandNonPortProtos(
-            'Term %s contains mixed uses of protocols with and without port '
-            'numbers.\nProtocols: %s' % (self.name, self.protocol))
 
     if self.ttl:
       if not _MIN_TTL <= self.ttl <= _MAX_TTL:
@@ -1512,6 +1529,7 @@ class VarType:
   ENCAPSULATE = 61
   FILTER_TERM = 62
   RESTRICT_ADDRESS_FAMILY = 63
+  PORT_MIRROR = 64
 
 
   def __init__(self, var_type, value):
@@ -1717,6 +1735,7 @@ tokens = (
     'PLATFORMEXCLUDE',
     'POLICER',
     'PORT',
+    'PORT_MIRROR',
     'PRECEDENCE',
     'PRIORITY',
     'PROTOCOL',
@@ -1797,6 +1816,7 @@ reserved = {
     'platform-exclude': 'PLATFORMEXCLUDE',
     'policer': 'POLICER',
     'port': 'PORT',
+    'port-mirror': 'PORT_MIRROR',
     'precedence': 'PRECEDENCE',
     'priority': 'PRIORITY',
     'protocol': 'PROTOCOL',
@@ -1976,6 +1996,7 @@ def p_term_spec(p):
                 | term_spec platform_spec
                 | term_spec policer_spec
                 | term_spec port_spec
+                | term_spec port_mirror_spec
                 | term_spec precedence_spec
                 | term_spec priority_spec
                 | term_spec prefix_list_spec
@@ -2084,6 +2105,11 @@ def p_encapsulate_spec(p):
   p[0] = VarType(VarType.ENCAPSULATE, p[4])
 
 
+def p_port_mirror_spec(p):
+  """ port_mirror_spec : PORT_MIRROR ':' ':' STRING """
+  p[0] = VarType(VarType.PORT_MIRROR, p[4])
+
+
 def p_icmp_type_spec(p):
   """ icmp_type_spec : ICMP_TYPE ':' ':' one_or_more_strings """
   p[0] = VarType(VarType.ICMP_TYPE, p[4])
