@@ -31,6 +31,9 @@ class Term(juniper.Term):
     super().__init__(*args, **kwargs)
     self.term.protocol = ['icmp6' if x == 'icmpv6' else x
                           for x in self.term.protocol]
+    self.term.protocol_except = [
+        'icmp6' if x == 'icmpv6' else x for x in self.term.protocol_except
+    ]
 
   def NormalizeIcmpTypes(self, icmp_types, protocols, af):
     protocols = ['icmpv6' if x == 'icmp6' else x for x in protocols]
@@ -54,4 +57,6 @@ class SRXlo(juniper.Juniper):
     supported_tokens.remove('flexible_match_range')
     # currently only support 'encapsulate' in juniper
     supported_tokens.remove('encapsulate')
+    # currently only support 'port-mirror' in juniper
+    supported_tokens.remove('port_mirror')
     return supported_tokens, supported_sub_tokens
