@@ -14,23 +14,18 @@
 
 """Unittest for juniper acl rendering module."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import datetime
 import re
-import unittest
+from absl.testing import absltest
+from unittest import mock
 
 from absl import logging
+from absl.testing import parameterized
 from capirca.lib import aclgenerator
 from capirca.lib import juniper
 from capirca.lib import nacaddr
 from capirca.lib import naming
 from capirca.lib import policy
-import mock
-from six.moves import range
 
 GOOD_HEADER = """
 header {
@@ -48,6 +43,11 @@ header {
   target:: juniper test-filter inet6
 }
 """
+GOOD_HEADER_MIXED = """
+header {
+  target:: juniper test-filter mixed
+}
+"""
 GOOD_HEADER_BRIDGE = """
 header {
   target:: juniper test-filter bridge
@@ -58,6 +58,11 @@ header {
   target:: juniper test-filter enable_dsmo
 }
 """
+GOOD_FILTER_ENHANCED_MODE_HEADER = """
+header {
+  target:: juniper test-filter filter_enhanced_mode
+}
+"""
 GOOD_NOVERBOSE_V4_HEADER = """
 header {
   target:: juniper test-filter inet noverbose
@@ -380,12 +385,25 @@ term good-term-36 {
   action:: accept
 }
 """
+GOOD_TERM_37 = """
+term good-term-37 {
+  destination-address:: SOME_HOST
+  restrict-address-family:: inet
+  action:: accept
+}
+"""
 GOOD_TERM_COMMENT = """
 term good-term-comment {
   comment:: "This is a COMMENT"
   action:: accept
 }
 """
+GOOD_TERM_FILTER = """
+term good-term-filter {
+  comment:: "This is a COMMENT"
+  filter-term:: my-filter
+}
+"""
 BAD_TERM_1 = """
 term bad-term-1 {
   protocol:: tcp udp
@@ -458,6 +476,20 @@ term bad-term-2 {
   routing-instance:: instance-name
 }
 """
+PORTMIRROR_GOOD_TERM_1 = """
+term good-term-1 {
+  protocol:: tcp
+  port-mirror:: true
+}
+"""
+PORTMIRROR_GOOD_TERM_2 = """
+term good-term-2 {
+  protocol:: tcp
+  port-mirror:: true
+  counter:: count-name
+  action:: deny
+}
+"""
 LONG_COMMENT_TERM_1 = """
 term long-comment-term-1 {
   comment:: "this is very very very very very very very very very very very
@@ -477,6 +509,12 @@ term good-term-1 {
   action:: accept
 }
 """
+HOPOPT_TERM_EXCEPT = """
+term good-term-1 {
+  protocol-except:: hopopt
+  action:: accept
+}
+"""
 FRAGOFFSET_TERM = """
 term good-term-1 {
   fragment-offset:: 1-7
@@ -538,6 +576,22 @@ term flex-match-term-1 {
   action:: deny
 }
 """
+BAD_TERM_FILTER = """
+term bad_term_filter {
+  filter-term:: my-filter
+  action:: deny
+}
+"""
+
+MIXED_TESTING_TERM = """
+term good-term {
+  protocol:: tcp
+  source-address:: SOME_HOST
+  destination-port:: SMTP
+  destination-address:: SOME_OTHER_HOST
+  action:: accept
+}
+"""
 
 SUPPORTED_TOKENS = frozenset([
     'action',
@@ -555,6 +609,7 @@ SUPPORTED_TOKENS = frozenset([
     'encapsulate',
     'ether_type',
     'expiration',
+    'filter_term',
     'flexible_match_range',
     'forwarding_class',
     'forwarding_class_except',
@@ -574,10 +629,12 @@ SUPPORTED_TOKENS = frozenset([
     'platform_exclude',
     'policer',
     'port',
+    'port_mirror',
     'precedence',
     'protocol',
     'protocol_except',
     'qos',
+    'restrict_address_family',
     'routing_instance',
     'source_address',
     'source_address_exclude',
@@ -651,10 +708,10 @@ SUPPORTED_SUB_TOKENS = {
 EXP_INFO = 2
 
 
-class JuniperTest(unittest.TestCase):
+class JuniperTest(parameterized.TestCase):
 
   def setUp(self):
-    super(JuniperTest, self).setUp()
+    super().setUp()
     self.naming = mock.create_autospec(naming.Naming)
 
   def testOptions(self):
@@ -757,6 +814,20 @@ class JuniperTest(unittest.TestCase):
         EXP_INFO)
     self.assertRaises(juniper.JuniperMultipleTerminatingActionError, str, jcl)
 
+  def testPortMirror(self):
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + PORTMIRROR_GOOD_TERM_1, self.naming),
+        EXP_INFO)
+    output = str(jcl)
+    self.assertIn('port-mirror;', output, output)
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + PORTMIRROR_GOOD_TERM_2, self.naming),
+        EXP_INFO)
+    output = str(jcl)
+    self.assertIn('port-mirror;', output, output)
+    self.assertIn('count count-name;', output, output)
+    self.assertIn('discard;', output, output)
+
   def testIcmpType(self):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER + GOOD_TERM_3,
                                              self.naming), EXP_INFO)
@@ -821,6 +892,19 @@ class JuniperTest(unittest.TestCase):
     self.naming.GetNetAddr.assert_called_once_with('SOME_HOST')
     self.naming.GetServiceByProto.assert_called_once_with('SMTP', 'tcp')
 
+  def testFilterEnhancedModeHeader(self):
+    self.naming.GetNetAddr.return_value = [nacaddr.IP('10.0.0.0/8')]
+    self.naming.GetServiceByProto.return_value = ['25']
+
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_FILTER_ENHANCED_MODE_HEADER + GOOD_TERM_1,
+                           self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertIn('enhanced-mode;', output, output)
+
+    self.naming.GetNetAddr.assert_called_once_with('SOME_HOST')
+    self.naming.GetServiceByProto.assert_called_once_with('SMTP', 'tcp')
+
   def testHopLimit(self):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER_V6 +
                                              GOOD_TERM_V6_HOP_LIMIT,
@@ -828,6 +912,13 @@ class JuniperTest(unittest.TestCase):
     output = str(jcl)
     self.assertIn('hop-limit 25;', output, output)
 
+  def testHopLimitInet(self):
+    jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER +
+                                             GOOD_TERM_V6_HOP_LIMIT,
+                                             self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertNotIn('hop-limit 25;', output, output)
+
   def testProtocolExcept(self):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER_V6 + GOOD_TERM_7,
                                              self.naming), EXP_INFO)
@@ -1016,6 +1107,31 @@ class JuniperTest(unittest.TestCase):
     self.naming.GetServiceByProto.assert_has_calls([
         mock.call('DNS', 'tcp'), mock.call('DNS', 'udp')])
 
+  def testMixedFilterInetType(self):
+    self.naming.GetNetAddr.return_value = [
+        nacaddr.IPv4('127.0.0.1'), nacaddr.IPv6('::1/128')]
+
+    jcl = juniper.Juniper(policy.ParsePolicy(
+        GOOD_HEADER_MIXED + GOOD_TERM_12, self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertIn('test-filter4', output, output)
+    self.assertIn('127.0.0.1', output, output)
+    self.assertIn('test-filter6', output, output)
+    self.assertIn('::1/128', output, output)
+
+    self.naming.GetNetAddr.assert_called_once_with('LOCALHOST')
+
+  def testRestrictAddressFamilyType(self):
+    self.naming.GetNetAddr.return_value = [
+        nacaddr.IPv4('127.0.0.1'), nacaddr.IPv6('::1/128')]
+
+    jcl = juniper.Juniper(policy.ParsePolicy(
+        GOOD_HEADER_MIXED + GOOD_TERM_37, self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertIn('127.0.0.1', output, output)
+    self.assertNotIn('::1/128', output, output)
+    self.naming.GetNetAddr.assert_called_once_with('SOME_HOST')
+
   def testBridgeFilterInetType(self):
     self.naming.GetNetAddr.return_value = [
         nacaddr.IPv4('127.0.0.1'), nacaddr.IPv6('::1/128')]
@@ -1144,6 +1260,16 @@ class JuniperTest(unittest.TestCase):
 
     self.naming.GetServiceByProto.assert_called_once_with('SSH', 'tcp')
 
+  def testFilterTerm(self):
+    jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER + GOOD_TERM_FILTER,
+                                             self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertIn('filter my-filter;', output, output)
+
+  def testFilterActionTerm(self):
+    with self.assertRaises(policy.InvalidTermActionError):
+      policy.ParsePolicy(GOOD_HEADER + BAD_TERM_FILTER, self.naming)
+
   def testArbitraryOptions(self):
     self.naming.GetServiceByProto.return_value = ['22']
 
@@ -1378,6 +1504,12 @@ class JuniperTest(unittest.TestCase):
     output = str(jcl)
     self.assertIn('ttl 10;', output)
 
+  def testTTLInet6(self):
+    jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER_V6 + GOOD_TERM_21,
+                                             self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertNotIn('ttl 10;', output)
+
   def testNextIpFormat(self):
     self.naming.GetNetAddr.return_value = [nacaddr.IP('10.1.1.1/32')]
 
@@ -1443,6 +1575,14 @@ class JuniperTest(unittest.TestCase):
                                              self.naming), EXP_INFO)
     output = str(jcl)
     self.assertIn('protocol hop-by-hop;', output, output)
+    self.assertNotIn('protocol hopopt;', output, output)
+
+  def testHopOptProtocolExcept(self):
+    jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER + HOPOPT_TERM_EXCEPT,
+                                             self.naming), EXP_INFO)
+    output = str(jcl)
+    self.assertIn('protocol-except hop-by-hop;', output, output)
+    self.assertNotIn('protocol-except hopopt;', output, output)
 
   def testFlexibleMatch(self):
     jcl = juniper.Juniper(policy.ParsePolicy(
@@ -1526,6 +1666,136 @@ class JuniperTest(unittest.TestCase):
                       GOOD_HEADER_V6 + BAD_FLEX_MATCH_TERM_4,
                       self.naming)
 
+  @parameterized.named_parameters(
+      ('MIXED_TO_V4',
+       [[nacaddr.IPv4('0.0.0.0/1'),
+         nacaddr.IPv6('2001::/33')], [nacaddr.IPv4('192.168.0.0/24')]], [
+             '            term good-term {\n' +
+             '                from {\n' +
+             '                    source-address {\n' +
+             '                        0.0.0.0/1;\n' +
+             '                    }\n' +
+             '                    destination-address {\n' +
+             '                        192.168.0.0/24;\n' +
+             '                    }'
+         ], ['2001::/33']),
+      ('V4_TO_MIXED', [
+          [nacaddr.IPv4('192.168.0.0/24')],
+          [nacaddr.IPv4('0.0.0.0/1'),
+           nacaddr.IPv6('2001::/33')],
+      ], [
+          '            term good-term {\n' +
+          '                from {\n' +
+          '                    source-address {\n' +
+          '                        192.168.0.0/24;\n' +
+          '                    }\n' +
+          '                    destination-address {\n' +
+          '                        0.0.0.0/1;\n' +
+          '                    }'
+      ], ['2001::/33']),
+      ('MIXED_TO_V6',
+       [[nacaddr.IPv4('0.0.0.0/1'),
+         nacaddr.IPv6('2001::/33')], [nacaddr.IPv6('2201::/48')]], [
+             '            term good-term {\n' +
+             '                from {\n' +
+             '                    source-address {\n' +
+             '                        2001::/33;\n' +
+             '                    }\n' +
+             '                    destination-address {\n' +
+             '                        2201::/48;\n' +
+             '                    }'
+         ], ['0.0.0.0/1']),
+      ('V6_TO_MIXED', [[
+          nacaddr.IPv6('2201::/48')
+      ], [nacaddr.IPv4('0.0.0.0/1'),
+          nacaddr.IPv6('2001::/33')]], [
+              '            term good-term {\n' +
+              '                from {\n' +
+              '                    source-address {\n' +
+              '                        2201::/48;\n' +
+              '                    }\n' +
+              '                    destination-address {\n' +
+              '                        2001::/33;\n' +
+              '                    }'
+        ], ['0.0.0.0/1']),
+      ('MIXED_TO_MIXED', [[
+          nacaddr.IPv4('0.0.0.0/1'),
+          nacaddr.IPv6('2001::/33')
+      ], [nacaddr.IPv4('192.168.0.0/24'),
+          nacaddr.IPv6('2201::/48')]], [
+              '            term good-term {\n' +
+              '                from {\n' +
+              '                    source-address {\n' +
+              '                        0.0.0.0/1;\n' +
+              '                    }\n' +
+              '                    destination-address {\n' +
+              '                        192.168.0.0/24;\n' +
+              '                    }',
+              '            term good-term {\n' +
+              '                from {\n' +
+              '                    source-address {\n' +
+              '                        2001::/33;\n' +
+              '                    }\n' +
+              '                    destination-address {\n' +
+              '                        2201::/48;\n' +
+              '                    }'
+        ], []),
+      ('V4_TO_V4', [[nacaddr.IPv4('0.0.0.0/1')],
+                    [nacaddr.IPv4('192.168.0.0/24')]], [
+                        '            term good-term {\n' +
+                        '                from {\n' +
+                        '                    source-address {\n' +
+                        '                        0.0.0.0/1;\n' +
+                        '                    }\n' +
+                        '                    destination-address {\n' +
+                        '                        192.168.0.0/24;\n' +
+                        '                    }'
+      ], []),
+      ('V6_TO_V6', [[nacaddr.IPv6('2001::/33')], [nacaddr.IPv6('2201::/48')]], [
+          '            term good-term {\n' +
+          '                from {\n' +
+          '                    source-address {\n' +
+          '                        2001::/33;\n' +
+          '                    }\n' +
+          '                    destination-address {\n' +
+          '                        2201::/48;\n' +
+          '                    }'
+      ], []),
+      (
+          'V4_TO_V6',
+          [[nacaddr.IPv4('0.0.0.0/1')], [nacaddr.IPv6('2201::/48')]],
+          [],
+          ['0.0.0.0/1', '192.168.0.0/24', '2001::/33', '2201::/48'],
+      ),
+      (
+          'V6_TO_V4',
+          [[nacaddr.IPv6('2001::/33')], [nacaddr.IPv4('192.168.0.0/24')]],
+          [],
+          ['0.0.0.0/1', '192.168.0.0/24', '2001::/33', '2201::/48'],
+      ),
+      (
+          'PARTLY_UNSPECIFIED',
+          [[nacaddr.IPv6('2001::/33')], [nacaddr.IPv4('192.168.0.0/24')]],
+          ['term good_term_25 '],
+          [
+              '0.0.0.0/1', '192.168.0.0/24', '2001::/33', '2201::/48',
+              'term good-term-both-icmp-and-icmpv6-'
+          ],
+      ),
+  )
+  def testMixed(self, addresses, expected, notexpected):
+    self.naming.GetNetAddr.side_effect = addresses
+    self.naming.GetServiceByProto.return_value = ['25']
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(
+            GOOD_HEADER_MIXED + MIXED_TESTING_TERM + GOOD_TERM_25, self.naming),
+        EXP_INFO)
+    output = str(jcl)
+    for expect in expected:
+      self.assertIn(expect, output, output)
+    for notexpect in notexpected:
+      self.assertNotIn(notexpect, output, output)
+
 
 if __name__ == '__main__':
-  unittest.main()
+  absltest.main()
