@@ -14,22 +14,17 @@
 
 """Unit test for Juniper SRX acl rendering module."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import copy
 import datetime
 import re
-import unittest
+from absl.testing import absltest
+from unittest import mock
 
 from capirca.lib import aclgenerator
 from capirca.lib import junipersrx
 from capirca.lib import nacaddr
 from capirca.lib import naming
 from capirca.lib import policy
-import mock
 
 
 GOOD_HEADER = """
@@ -337,6 +332,12 @@ term good_term_21 {
 }
 """
 
+GOOD_TERM_23 = """
+term good_term_23 {
+  action:: accept
+}
+"""
+
 BAD_TERM_1 = """
 term bad-term-1 {
   destination-address:: SOME_HOST
@@ -366,6 +367,14 @@ term udp-established-term {
 }
 """
 
+ICMP_RESPONSE_TERM = """
+term icmp_response-term {
+  protocol:: icmp
+  icmp-type:: echo-reply
+  action:: accept
+}
+"""
+
 EXPIRED_TERM_1 = """
 term expired_test {
   expiration:: 2000-1-1
@@ -497,6 +506,31 @@ term timeout-term {
 }
 """
 
+PLATFORM_EXCLUDE_TERM = """
+term platform-exclude-term {
+  protocol:: tcp udp
+  platform-exclude:: srx
+  action:: accept
+}
+"""
+
+PLATFORM_TERM = """
+term platform-term {
+  protocol:: tcp udp
+  platform:: srx juniper
+  action:: accept
+}
+"""
+
+PLATFORM_EXCLUDE_ADDRESS_TERM = """
+term platform-exclude-term {
+  protocol:: tcp udp
+  source-address:: FOO
+  platform-exclude:: srx
+  action:: accept
+}
+"""
+
 SUPPORTED_TOKENS = {
     'action',
     'comment',
@@ -585,10 +619,10 @@ _IPSET4 = [nacaddr.IP('10.0.0.0/20')]
 _IPSET5 = [nacaddr.IP('10.0.0.0/24')]
 
 
-class JuniperSRXTest(unittest.TestCase):
+class JuniperSRXTest(absltest.TestCase):
 
   def setUp(self):
-    super(JuniperSRXTest, self).setUp()
+    super().setUp()
     self.naming = mock.create_autospec(naming.Naming)
 
   def testHeaderComment(self):
@@ -1312,6 +1346,51 @@ class JuniperSRXTest(unittest.TestCase):
         'address GOOD_TERM_19_SRC_EXCLUDE_3 10.0.8.0/21;', output, output)
     self.assertNotIn('10.0.0.0/24', output)
 
+  def testPlatformExclude(self):
+    large = [nacaddr.IP('10.0.0.0/20', 'LARGE', 'LARGE')]
+    small = [nacaddr.IP('10.0.0.0/24', 'SMALL', 'SMALL')]
+    self.naming.GetNetAddr.side_effect = [large, small]
+
+    pol = policy.ParsePolicy(GOOD_HEADER + PLATFORM_EXCLUDE_TERM + GOOD_TERM_19,
+                             self.naming)
+    output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
+    self.assertIn('good_term_19', output,
+                  output)
+    self.assertNotIn('platform-exclude-term', output)
+
+  def testPlatformTerm(self):
+    large = [nacaddr.IP('10.0.0.0/20', 'LARGE', 'LARGE')]
+    small = [nacaddr.IP('10.0.0.0/24', 'SMALL', 'SMALL')]
+    self.naming.GetNetAddr.side_effect = [large, small]
+
+    pol = policy.ParsePolicy(GOOD_HEADER + PLATFORM_TERM + GOOD_TERM_19,
+                             self.naming)
+    output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
+    self.assertIn('good_term_19', output,
+                  output)
+    self.assertIn('platform-term', output, output)
+
+  def testPlatformExcludeWithSourceExclude(self):
+    foo = [nacaddr.IP('192.1.0.0/20', 'FOO', 'FOO')]
+    large = [nacaddr.IP('10.0.0.0/20', 'LARGE', 'LARGE')]
+    small = [nacaddr.IP('10.0.0.0/24', 'SMALL', 'SMALL')]
+    self.naming.GetNetAddr.side_effect = [foo, large, small]
+
+    pol = policy.ParsePolicy(
+        GOOD_HEADER + PLATFORM_EXCLUDE_ADDRESS_TERM + GOOD_TERM_19, self.naming)
+    output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
+    self.assertIn('address GOOD_TERM_19_SRC_EXCLUDE_0 10.0.1.0/24;', output,
+                  output)
+    self.assertIn('address GOOD_TERM_19_SRC_EXCLUDE_1 10.0.2.0/23;', output,
+                  output)
+    self.assertIn('address GOOD_TERM_19_SRC_EXCLUDE_2 10.0.4.0/22;', output,
+                  output)
+    self.assertIn('address GOOD_TERM_19_SRC_EXCLUDE_3 10.0.8.0/21;', output,
+                  output)
+    self.assertNotIn('10.0.0.0/24', output)
+    self.assertNotIn('192.1.0.0/20', output)
+    self.assertNotIn('platform-exclude-term', output)
+
   def testMixedVersionIcmp(self):
     pol = policy.ParsePolicy(GOOD_HEADER + ICMP_TYPE_TERM_1 + IPV6_ICMP_TERM,
                              self.naming)
@@ -1364,6 +1443,24 @@ class JuniperSRXTest(unittest.TestCase):
     self.assertNotIn('udp-established-term', output)
     self.assertNotIn('tcp-established-term', output)
 
+  def testStatelessReply(self):
+    self.naming.GetNetAddr.return_value = _IPSET
+    self.naming.GetServiceByProto.return_value = ['25']
+
+    ret = policy.ParsePolicy(GOOD_HEADER + GOOD_TERM_1 + ICMP_RESPONSE_TERM,
+                             self.naming)
+
+    _, terms = ret.filters[0]
+    for term in terms:
+      if term.protocol[0] == 'icmp':
+        term.stateless_reply = True
+
+    srx = junipersrx.JuniperSRX(ret, EXP_INFO)
+
+    output = str(srx)
+    self.assertIn('policy good-term-1 {', output, output)
+    self.assertNotIn('policy icmp_response-term {', output, output)
+
   def testNoVerbose(self):
     self.naming.GetNetAddr.return_value = _IPSET
     self.naming.GetServiceByProto.return_value = ['25']
@@ -1803,5 +1900,18 @@ class JuniperSRXTest(unittest.TestCase):
         # once i.e. is defined in the address book
         self.assertGreater(address_set_count, 1)
 
+  def testEmptyApplications(self):
+    self.naming.GetNetAddr.return_value = _IPSET
+
+    # GOOD_HEADER_3 doesn't matter, any valid header should do
+    pol = policy.ParsePolicy(GOOD_HEADER_3 + GOOD_TERM_23,
+                             self.naming)
+    p = junipersrx.JuniperSRX(pol, EXP_INFO)
+    output = p._GenerateApplications()
+
+    pattern = re.compile(r'delete: applications;')
+    self.assertTrue(pattern.search(str(''.join(output))), ''.join(output))
+
+
 if __name__ == '__main__':
-  unittest.main()
+  absltest.main()
