@@ -14,18 +14,13 @@
 
 """Unit tests for policy.py library."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
-import unittest
+from absl.testing import absltest
+from unittest import mock
 
 from absl import logging
 from capirca.lib import nacaddr
 from capirca.lib import naming
 from capirca.lib import policy
-import mock
 
 
 HEADER = """
@@ -430,6 +425,12 @@ term good-term-46 {
   encapsulate:: stuff_and_things
 }
 """
+GOOD_TERM_47 = """
+term good-term-47 {
+  protocol:: icmp tcp udp gre esp ah sctp
+  port-mirror:: true
+}
+"""
 GOOD_TERM_V6_1 = """
 term good-term-v6-1 {
   hop-limit:: 5
@@ -579,10 +580,10 @@ term bad-term-16 {
 # pylint: disable=maybe-no-member
 
 
-class PolicyTest(unittest.TestCase):
+class PolicyTest(absltest.TestCase):
 
   def setUp(self):
-    super(PolicyTest, self).setUp()
+    super().setUp()
     self.naming = mock.create_autospec(naming.Naming)
 
   @mock.patch.object(policy, '_ReadFile')
@@ -1236,6 +1237,11 @@ class PolicyTest(unittest.TestCase):
     result = policy.ParsePolicy(pol, self.naming)
     self.assertIn('encapsulate: stuff_and_things', str(result))
 
+  def testPortMirror(self):
+    pol = HEADER + GOOD_TERM_47
+    result = policy.ParsePolicy(pol, self.naming)
+    self.assertIn('port_mirror: true', str(result))
+
   def testTTL(self):
     pol = HEADER + GOOD_TERM_43
     result = policy.ParsePolicy(pol, self.naming)
@@ -1687,4 +1693,4 @@ class PolicyTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
-  unittest.main()
+  absltest.main()
